from nats.aio.client import Client as Nats
from nats.errors import TimeoutError
import json
import asyncio

from inference.file_writer import FileWriter
class NatsListener:
    def __init__(self, file_writer: FileWriter, nats_uri: str = 'nats://localhost:4222', 
                       subject: str = 'inference.result.profile'):
        self.nc = Nats()

        self.nats_uri = nats_uri
        self.subject = subject
        self.file_writer = file_writer
    
    async def connect(self):
        await self.nc.connect(self.nats_uri)
        self.sub = await self.nc.subscribe(self.subject)

    async def record(self) -> None:
        while not self.file_writer.is_closed():
            try:
                msg = await self.sub.next_msg(timeout=10)
            except TimeoutError as e:
                await asyncio.sleep(0)
                continue
            except Exception as e:
                print(e)
                break

            msg = json.loads(msg.data)
            self.file_writer.write(str(msg))



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Empty(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class PipelineReq(_message.Message):
    __slots__ = ["app_id"]
    APP_ID_FIELD_NUMBER: _ClassVar[int]
    app_id: str
    def __init__(self, app_id: _Optional[str] = ...) -> None: ...

class PipelineRes(_message.Message):
    __slots__ = ["result"]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    result: bool
    def __init__(self, result: bool = ...) -> None: ...

class InferenceReq(_message.Message):
    __slots__ = ["app_id", "stream_id", "uri", "settings", "name", "offset"]
    APP_ID_FIELD_NUMBER: _ClassVar[int]
    STREAM_ID_FIELD_NUMBER: _ClassVar[int]
    URI_FIELD_NUMBER: _ClassVar[int]
    SETTINGS_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    app_id: str
    stream_id: str
    uri: str
    settings: str
    name: str
    offset: int
    def __init__(self, app_id: _Optional[str] = ..., stream_id: _Optional[str] = ..., uri: _Optional[str] = ..., settings: _Optional[str] = ..., name: _Optional[str] = ..., offset: _Optional[int] = ...) -> None: ...

class InferenceRes(_message.Message):
    __slots__ = ["count", "status", "eos", "err", "meta", "snapshot", "app_id", "stream_id"]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    EOS_FIELD_NUMBER: _ClassVar[int]
    ERR_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    APP_ID_FIELD_NUMBER: _ClassVar[int]
    STREAM_ID_FIELD_NUMBER: _ClassVar[int]
    count: int
    status: int
    eos: bool
    err: bool
    meta: str
    snapshot: bytes
    app_id: str
    stream_id: str
    def __init__(self, count: _Optional[int] = ..., status: _Optional[int] = ..., eos: bool = ..., err: bool = ..., meta: _Optional[str] = ..., snapshot: _Optional[bytes] = ..., app_id: _Optional[str] = ..., stream_id: _Optional[str] = ...) -> None: ...

class AppReq(_message.Message):
    __slots__ = ["app_id", "chunk"]
    APP_ID_FIELD_NUMBER: _ClassVar[int]
    CHUNK_FIELD_NUMBER: _ClassVar[int]
    app_id: str
    chunk: bytes
    def __init__(self, app_id: _Optional[str] = ..., chunk: _Optional[bytes] = ...) -> None: ...

class AppRes(_message.Message):
    __slots__ = ["result"]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    result: bool
    def __init__(self, result: bool = ...) -> None: ...

class AppList(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _containers.RepeatedCompositeFieldContainer[App]
    def __init__(self, app: _Optional[_Iterable[_Union[App, _Mapping]]] = ...) -> None: ...

class InferenceList(_message.Message):
    __slots__ = ["inference"]
    INFERENCE_FIELD_NUMBER: _ClassVar[int]
    inference: _containers.RepeatedCompositeFieldContainer[InferenceReq]
    def __init__(self, inference: _Optional[_Iterable[_Union[InferenceReq, _Mapping]]] = ...) -> None: ...

class InferenceResList(_message.Message):
    __slots__ = ["inference"]
    INFERENCE_FIELD_NUMBER: _ClassVar[int]
    inference: _containers.RepeatedCompositeFieldContainer[InferenceRes]
    def __init__(self, inference: _Optional[_Iterable[_Union[InferenceRes, _Mapping]]] = ...) -> None: ...

class LicReq(_message.Message):
    __slots__ = ["key", "hash_code"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    HASH_CODE_FIELD_NUMBER: _ClassVar[int]
    key: str
    hash_code: str
    def __init__(self, key: _Optional[str] = ..., hash_code: _Optional[str] = ...) -> None: ...

class LicRes(_message.Message):
    __slots__ = ["result", "hash_code"]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    HASH_CODE_FIELD_NUMBER: _ClassVar[int]
    result: bool
    hash_code: str
    def __init__(self, result: bool = ..., hash_code: _Optional[str] = ...) -> None: ...

class StreamingReq(_message.Message):
    __slots__ = ["uri", "session_id"]
    URI_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    uri: str
    session_id: str
    def __init__(self, uri: _Optional[str] = ..., session_id: _Optional[str] = ...) -> None: ...

class StreamingRes(_message.Message):
    __slots__ = ["location", "ts_start", "session_id"]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    TS_START_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    location: str
    ts_start: int
    session_id: str
    def __init__(self, location: _Optional[str] = ..., ts_start: _Optional[int] = ..., session_id: _Optional[str] = ...) -> None: ...

class Model(_message.Message):
    __slots__ = ["id", "name", "version", "platform", "framework", "capacity", "precision", "desc", "path", "compute_capa", "ref_count"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    PLATFORM_FIELD_NUMBER: _ClassVar[int]
    FRAMEWORK_FIELD_NUMBER: _ClassVar[int]
    CAPACITY_FIELD_NUMBER: _ClassVar[int]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    DESC_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    COMPUTE_CAPA_FIELD_NUMBER: _ClassVar[int]
    REF_COUNT_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    version: str
    platform: str
    framework: str
    capacity: int
    precision: str
    desc: str
    path: str
    compute_capa: str
    ref_count: int
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., version: _Optional[str] = ..., platform: _Optional[str] = ..., framework: _Optional[str] = ..., capacity: _Optional[int] = ..., precision: _Optional[str] = ..., desc: _Optional[str] = ..., path: _Optional[str] = ..., compute_capa: _Optional[str] = ..., ref_count: _Optional[int] = ...) -> None: ...

class Event(_message.Message):
    __slots__ = ["category_id", "category_name", "targets"]
    class Target(_message.Message):
        __slots__ = ["event_name", "event_id", "classes"]
        EVENT_NAME_FIELD_NUMBER: _ClassVar[int]
        EVENT_ID_FIELD_NUMBER: _ClassVar[int]
        CLASSES_FIELD_NUMBER: _ClassVar[int]
        event_name: str
        event_id: str
        classes: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, event_name: _Optional[str] = ..., event_id: _Optional[str] = ..., classes: _Optional[_Iterable[str]] = ...) -> None: ...
    CATEGORY_ID_FIELD_NUMBER: _ClassVar[int]
    CATEGORY_NAME_FIELD_NUMBER: _ClassVar[int]
    TARGETS_FIELD_NUMBER: _ClassVar[int]
    category_id: str
    category_name: str
    targets: _containers.RepeatedCompositeFieldContainer[Event.Target]
    def __init__(self, category_id: _Optional[str] = ..., category_name: _Optional[str] = ..., targets: _Optional[_Iterable[_Union[Event.Target, _Mapping]]] = ...) -> None: ...

class Pipeline(_message.Message):
    __slots__ = ["name", "plugin", "model_id", "classes"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PLUGIN_FIELD_NUMBER: _ClassVar[int]
    MODEL_ID_FIELD_NUMBER: _ClassVar[int]
    CLASSES_FIELD_NUMBER: _ClassVar[int]
    name: str
    plugin: str
    model_id: str
    classes: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, name: _Optional[str] = ..., plugin: _Optional[str] = ..., model_id: _Optional[str] = ..., classes: _Optional[_Iterable[str]] = ...) -> None: ...

class App(_message.Message):
    __slots__ = ["id", "name", "evgen_path", "models", "events", "pipelines", "cover_path", "desc", "memory_usage", "version", "framework", "outputs"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    EVGEN_PATH_FIELD_NUMBER: _ClassVar[int]
    MODELS_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    PIPELINES_FIELD_NUMBER: _ClassVar[int]
    COVER_PATH_FIELD_NUMBER: _ClassVar[int]
    DESC_FIELD_NUMBER: _ClassVar[int]
    MEMORY_USAGE_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    FRAMEWORK_FIELD_NUMBER: _ClassVar[int]
    OUTPUTS_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    evgen_path: str
    models: _containers.RepeatedCompositeFieldContainer[Model]
    events: str
    pipelines: str
    cover_path: str
    desc: str
    memory_usage: int
    version: str
    framework: str
    outputs: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., evgen_path: _Optional[str] = ..., models: _Optional[_Iterable[_Union[Model, _Mapping]]] = ..., events: _Optional[str] = ..., pipelines: _Optional[str] = ..., cover_path: _Optional[str] = ..., desc: _Optional[str] = ..., memory_usage: _Optional[int] = ..., version: _Optional[str] = ..., framework: _Optional[str] = ..., outputs: _Optional[str] = ...) -> None: ...

class Dx(_message.Message):
    __slots__ = ["id", "name", "address", "capacity", "activated", "version", "framework", "lic_type", "lic_end_date", "lic_key"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    CAPACITY_FIELD_NUMBER: _ClassVar[int]
    ACTIVATED_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    FRAMEWORK_FIELD_NUMBER: _ClassVar[int]
    LIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    LIC_END_DATE_FIELD_NUMBER: _ClassVar[int]
    LIC_KEY_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    address: str
    capacity: int
    activated: int
    version: str
    framework: str
    lic_type: str
    lic_end_date: str
    lic_key: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ..., address: _Optional[str] = ..., capacity: _Optional[int] = ..., activated: _Optional[int] = ..., version: _Optional[str] = ..., framework: _Optional[str] = ..., lic_type: _Optional[str] = ..., lic_end_date: _Optional[str] = ..., lic_key: _Optional[str] = ...) -> None: ...



from proto_py.dx_pb2_grpc import DetectorStub
from proto_py.dx_pb2 import Empty, AppReq, InferenceReq, Dx

import grpc
from asyncio import sleep

BUF_SIZE = 4096


def file_iterator(app_id: str, file: str):
    with open(file, 'rb') as f:
        while True:
            buf = f.read(BUF_SIZE)
            if len(buf) == 0:
                return
            yield AppReq(app_id=app_id, chunk=buf)


class InferenceDX:
    def __init__(self) -> None:
        self.channel = grpc.insecure_channel('localhost:50051')
        self.stub = DetectorStub(self.channel)
        self.inf_req = None

    def ping(self) -> None:
        response: Dx = self.stub.GetDx(Empty())
        print(f'Connected to {response.name}')

    def install_app(self, app_id: str, file: str) -> None:
        read_file_iterator = file_iterator(app_id, file)

        response = self.stub.InstallApp(read_file_iterator)

    async def start_inference(self, app_id: str, stream_id: str, uri: str, timestamp: int) -> None:
        # Add inference
        self.inf_req = InferenceReq(
            app_id=app_id, stream_id=stream_id, uri=uri, offset=timestamp)
        result = self.stub.AddInference(self.inf_req)
        print(f'Add inference: {result}')

        while True:
            result = self.stub.GetInferenceStatus(self.inf_req)
            if result.eos or result.err:
                break
            print('Waiting for EOS...')
            await sleep(3)

        result = self.stub.RemoveInference(self.inf_req)
        print(f'Remove Inference: {result}')
        self.inf_req = None

    def clean_up_inference(self) -> None:
        if self.inf_req:
            self.stub.RemoveInference(self.inf_req)
            self.inf_req = None

    def __del__(self) -> None:
        self.channel.close()



import sys
from collections import defaultdict

import cv2
from PyQt5.QtCore import Qt, QTimer
from PyQt5.QtGui import QImage, QPixmap, QPainter, QPen
from PyQt5.QtWidgets import QApplication, QMainWindow, QLabel, QDesktopWidget, \
QProgressBar, QHBoxLayout, QVBoxLayout, QWidget, QPushButton


VIDEO_EXTENSIONS = [".avi", ".mp4", ".mov"]


class VideoPlayer(QMainWindow):
    """Dx Video Player

    동영상 파일에 record.txt을 통해, 후처리를 한 Video를 관리하는 클래스.
    Space-bar로 동영상 재생 및 정지, Q/E 를 통해 Frame 이동 등의 기능을 제공한다.

    Attributes:
        cap (VideoCapture): cv2로 생성한 VideoCapture 객체.
        fps (int): cap에서 획득한 FPS(FramePerSeconds).
        total_frames (int): cap에서 획득한 total_frames
        current_frame (int): 현재 video_label에 그려지는 frame_number.
        
        video_label (QLabel): 후처리 된 영상을 출력하는 라벨.
        progress_bar (QProgressBar): 현재 프레임 / 전체 프레임을 보여주는 진행바.
        start_stop_button (QPushButton): 재생/정지 기능을 하는 toggle 버튼.

        timer (QTimer): fps에 맞게 video_label을 업데이트 하는 인터럽트 타이머.
        playing (bool): 동영상이 재생이면 True, 정지면 False.
        scaled_width (int): resize 된  video label의 width.
        scaled_height (int): resize 된 video_label의 height.
        data (List[Dict]): dict 형태의 record 정보가 들어있는 data list.
                        bbox, id, label, classifier 등이 존재.
    """

    def __init__(self, video_path: str, record: str, st_time_stamp: int) -> None:
        """
        Args:
            video_path: 동영상 파일의 경로
            record: object message의 내용
            st_time_stamp: 동영상 시작 시간의 time-stamp ex) 1690210800

        Raises:
            OpenError: 파일을 열 수 없음
            TimeError: st_time_stamp가 유효하지 않음 
        """
        super().__init__()

        file_ext = video_path[video_path.rfind("."):]
        # 파일 확장자가 영상 확장자인지 확인.
        if file_ext not in VIDEO_EXTENSIONS:
            raise Exception # OpenError : 동영상 확장자를 가지지 않는 파일

        #! cv2.VideoCapture에서 발생한 error는 except로 잡히지 않는다..
        self.cap = cv2.VideoCapture(video_path)

        if not self.cap.isOpened():
            raise Exception # OpenError : 동영상이 열리지 않음
        
        self.fps = int(self.cap.get(cv2.CAP_PROP_FPS))
        self.total_frames = int(self.cap.get(cv2.CAP_PROP_FRAME_COUNT))

        self.video_label = QLabel(self)
        self.video_label.setAlignment(Qt.AlignCenter)

        self.progress_bar = QProgressBar(self)
        self.progress_bar.setTextVisible(False)
        self.progress_bar.setMaximum(self.total_frames)
        self.progress_bar.setValue(0)

        self.start_stop_button = QPushButton("II", self)
        self.start_stop_button.clicked.connect(self.toggle_video)
        self.start_stop_button.setGeometry(10, 10, 100, 30)

        button_layout = QHBoxLayout()
        button_layout.addWidget(self.start_stop_button)
        button_layout.addWidget(self.progress_bar)

        main_layout = QVBoxLayout()
        main_layout.addWidget(self.video_label)
        main_layout.addLayout(button_layout)

        container = QWidget()
        container.setLayout(main_layout)
        self.setCentralWidget(container)

        self.playing = True 

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.update_frame)
        self.timer.start(1000 // self.fps)

        desktop = QDesktopWidget().screenGeometry()
        self.setMinimumSize(800, 600)
        self.setMaximumSize(desktop.width(), desktop.height())

        self.current_frame = 0 
        self.data = defaultdict(list)

        # 나노 시간 처리 후에, 시작 time_stamp 값을 빼서 초(second)를 구합니다.
        # 초(second)에다가 fps 값을 곱해 frame_number를 구합니다.
        for line in record:
            line_data = eval(line)
            time_in_seconds = (line_data["metadata"]["timestamp"] / 1e9) - st_time_stamp
            frame_num = int(time_in_seconds * self.fps)
            self.data[frame_num].append(line_data["objects"])

    def toggle_video(self):
        # video의 상태를 전환. 정지 -> 재생, 재생 -> 정지
        # 재생 -> 정지 시에는 timer는 정지.
        # 정지 -> 재생 시에는 timer를 다시 시작.
        if self.playing:
            self.timer.stop()
            self.start_stop_button.setText("▶")
        
        else:
            self.timer.start(1000 // self.fps)
            self.start_stop_button.setText("II")
        
        self.playing = not self.playing

    def update_frame(self):
        # frame을 하나 읽어들임
        # 현재 gui window의 size를 받아들여 frame을 resize.
                            # current_frame 값에 있는 data를 가져와서 id, label등을 draw.
        # frame 업데이트.

        ret, frame = self.cap.read()
        self.current_frame += 1

        if ret:
            frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
            h, w, ch = frame.shape

            # GUI 창의 크기를 가져와서 비율에 맞게 영상 크기를 조절
            window_width = self.video_label.width()
            window_height = self.video_label.height()
            aspect_ratio = w / h

            if window_width / window_height > aspect_ratio:
                self.scaled_width = int(window_height * aspect_ratio)
                self.scaled_height = window_height
            
            else:
                self.scaled_width = window_width
                self.scaled_height = int(window_width / aspect_ratio)

            frame = cv2.resize(frame, (self.scaled_width, self.scaled_height))

            bytes_per_line = ch * self.scaled_width
            q_image = QImage(frame.data, self.scaled_width, self.scaled_height, bytes_per_line, QImage.Format_RGB888)
            
            painter = QPainter(q_image)
            red_pen = QPen(Qt.red, 3)
            green_pen = QPen(Qt.green, 2)
            
            if self.current_frame in self.data: # 현재 frame에 해당하는 값의 정보들을 draw
                data = self.data[self.current_frame]

                for datum in data:
                    datum = datum[0]
                    painter.setPen(red_pen)
                    
                    bbox = datum["bbox"]
                    x, y, w, h = bbox["x"], bbox["y"], bbox["w"], bbox["h"]
                    x, y, w, h = x * self.scaled_width, y * self.scaled_height, \
                                    w * self.scaled_width, h * self.scaled_height

                    if datum["id"] >= 10000:
                        datum["id"] = -1

                    x, y, w, h = map(int, [x, y, w, h])
                    painter.drawRect(x, y, w, h)
                    
                    if datum["id"] != -1:
                        painter.drawText(x, y-10, f"[{datum['id']}] {datum['label']}")
                    else:
                        painter.drawText(x, y-10, f"[None] {datum['label']}")

                    painter.setPen(green_pen)

                    drawing_texts = []

                    if "classifiers" in datum:
                        for idx, val in enumerate(datum["classifiers"]):
                            drawing_texts.append(f"[{val['label']}] {val['type']}")
                    
                    if "score_d" in datum:
                            drawing_texts.append(f"Score: {datum['score_d']:.2f}")
                    
                    for idx, val in enumerate(drawing_texts):
                        painter.drawText(x, y + (idx+1)*15, val)

            painter.end()
            pixmap = QPixmap.fromImage(q_image)
            self.video_label.setPixmap(pixmap)
            self.progress_bar.setValue(self.current_frame)

        else:
            # exit
            self.timer.stop()
            self.cap.release()
            self.close()

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Space:
            self.toggle_video()
        
        elif event.key() == Qt.Key_Q:  # 이전 프레임으로 이동
            self.current_frame -= 2
            self.cap.set(cv2.CAP_PROP_POS_FRAMES, self.current_frame)
            self.update_frame()

        elif event.key() == Qt.Key_E:  # 다음 프레임으로 이동
            self.update_frame()

    def closeEvent(self, event):
        self.cap.release()
        self.timer.stop()


def text_input_player(text: str):
    """text 파일로 Videoplayer에 입력.
    
    첫 번째 줄, 영상 파일 경로
    두 번째 줄, timestamp (second)
    세번 째 줄 ~, 오브젝트 메시지

    Args:
        text: 위에 내용으로 구성된 text
    """
    with open(text, "r") as f:
        video_path = f.readline().strip()
        st_time_stamp = int(f.readline().strip())
        record = f.readlines()
    
    return VideoPlayer(video_path, record, st_time_stamp)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    player = text_input_player("sample.txt")
    player.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x08\x64x.proto\x12\x08\x61utocare\"\x07\n\x05\x45mpty\"\x1d\n\x0bPipelineReq\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\t\"\x1d\n\x0bPipelineRes\x12\x0e\n\x06result\x18\x01 \x01(\x08\"\xbe\x01\n\x0cInferenceReq\x12\x0e\n\x06\x61pp_id\x18\x01 \x01(\t\x12\x16\n\tstream_id\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03uri\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x15\n\x08settings\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x11\n\x04name\x18\x05 \x01(\tH\x03\x88\x01\x01\x12\x13\n\x06offset\x18\x06 \x01(\x03H\x04\x88\x01\x01\x42\x0c\n\n_stream_idB\x06\n\x04_uriB\x0b\n\t_settingsB\x07\n\x05_nameB\t\n\x07_offset\"\xf7\x01\n\x0cInferenceRes\x12\r\n\x05\x63ount\x18\x01 \x01(\x05\x12\x13\n\x06status\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x10\n\x03\x65os\x18\x03 \x01(\x08H\x01\x88\x01\x01\x12\x10\n\x03\x65rr\x18\x04 \x01(\x08H\x02\x88\x01\x01\x12\x11\n\x04meta\x18\x05 \x01(\tH\x03\x88\x01\x01\x12\x15\n\x08snapshot\x18\x06 \x01(\x0cH\x04\x88\x01\x01\x12\x13\n\x06\x61pp_id\x18\x07 \x01(\tH\x05\x88\x01\x01\x12\x16\n\tstream_id\x18\x08 \x01(\tH\x06\x88\x01\x01\x42\t\n\x07_statusB\x06\n\x04_eosB\x06\n\x04_errB\x07\n\x05_metaB\x0b\n\t_snapshotB\t\n\x07_app_idB\x0c\n\n_stream_id\"F\n\x06\x41ppReq\x12\x13\n\x06\x61pp_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05\x63hunk\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\t\n\x07_app_idB\x08\n\x06_chunk\"\x18\n\x06\x41ppRes\x12\x0e\n\x06result\x18\x01 \x01(\x08\"%\n\x07\x41ppList\x12\x1a\n\x03\x61pp\x18\x01 \x03(\x0b\x32\r.autocare.App\":\n\rInferenceList\x12)\n\tinference\x18\x01 \x03(\x0b\x32\x16.autocare.InferenceReq\"=\n\x10InferenceResList\x12)\n\tinference\x18\x01 \x03(\x0b\x32\x16.autocare.InferenceRes\"H\n\x06LicReq\x12\x10\n\x03key\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\thash_code\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x06\n\x04_keyB\x0c\n\n_hash_code\">\n\x06LicRes\x12\x0e\n\x06result\x18\x01 \x01(\x08\x12\x16\n\thash_code\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_hash_code\"P\n\x0cStreamingReq\x12\x10\n\x03uri\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nsession_id\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x06\n\x04_uriB\r\n\x0b_session_id\"~\n\x0cStreamingRes\x12\x15\n\x08location\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x15\n\x08ts_start\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x17\n\nsession_id\x18\x03 \x01(\tH\x02\x88\x01\x01\x42\x0b\n\t_locationB\x0b\n\t_ts_startB\r\n\x0b_session_id\"\xd4\x01\n\x05Model\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x10\n\x08platform\x18\x04 \x01(\t\x12\x11\n\tframework\x18\x05 \x01(\t\x12\x10\n\x08\x63\x61pacity\x18\x06 \x01(\x05\x12\x11\n\tprecision\x18\x07 \x01(\t\x12\x0c\n\x04\x64\x65sc\x18\x08 \x01(\t\x12\x0c\n\x04path\x18\t \x01(\t\x12\x14\n\x0c\x63ompute_capa\x18\n \x01(\t\x12\x16\n\tref_count\x18\x0b \x01(\x05H\x00\x88\x01\x01\x42\x0c\n\n_ref_count\"\x9d\x01\n\x05\x45vent\x12\x13\n\x0b\x63\x61tegory_id\x18\x01 \x01(\t\x12\x15\n\rcategory_name\x18\x02 \x01(\t\x12\'\n\x07targets\x18\x03 \x03(\x0b\x32\x16.autocare.Event.Target\x1a?\n\x06Target\x12\x12\n\nevent_name\x18\x01 \x01(\t\x12\x10\n\x08\x65vent_id\x18\x02 \x01(\t\x12\x0f\n\x07\x63lasses\x18\x03 \x03(\t\"]\n\x08Pipeline\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06plugin\x18\x02 \x01(\t\x12\x15\n\x08model_id\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x07\x63lasses\x18\x04 \x03(\tB\x0b\n\t_model_id\"\xe7\x02\n\x03\x41pp\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x17\n\nevgen_path\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x1f\n\x06models\x18\x04 \x03(\x0b\x32\x0f.autocare.Model\x12\x13\n\x06\x65vents\x18\x05 \x01(\tH\x01\x88\x01\x01\x12\x11\n\tpipelines\x18\x06 \x01(\t\x12\x17\n\ncover_path\x18\x07 \x01(\tH\x02\x88\x01\x01\x12\x0c\n\x04\x64\x65sc\x18\x08 \x01(\t\x12\x19\n\x0cmemory_usage\x18\t \x01(\x05H\x03\x88\x01\x01\x12\x14\n\x07version\x18\n \x01(\tH\x04\x88\x01\x01\x12\x16\n\tframework\x18\x0b \x01(\tH\x05\x88\x01\x01\x12\x14\n\x07outputs\x18\x0c \x01(\tH\x06\x88\x01\x01\x42\r\n\x0b_evgen_pathB\t\n\x07_eventsB\r\n\x0b_cover_pathB\x0f\n\r_memory_usageB\n\n\x08_versionB\x0c\n\n_frameworkB\n\n\x08_outputs\"\xb1\x01\n\x02\x44x\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61pacity\x18\x04 \x01(\x05\x12\x11\n\tactivated\x18\x05 \x01(\x05\x12\x0f\n\x07version\x18\x06 \x01(\t\x12\x11\n\tframework\x18\x07 \x01(\t\x12\x10\n\x08lic_type\x18\x08 \x01(\t\x12\x14\n\x0clic_end_date\x18\t \x01(\t\x12\x0f\n\x07lic_key\x18\n \x01(\t2\xb6\x08\n\x08\x44\x65tector\x12@\n\x0c\x41\x64\x64Inference\x12\x16.autocare.InferenceReq\x1a\x16.autocare.InferenceRes\"\x00\x12\x43\n\x0fRemoveInference\x12\x16.autocare.InferenceReq\x1a\x16.autocare.InferenceRes\"\x00\x12:\n\x12RemoveInferenceAll\x12\x10.autocare.AppReq\x1a\x10.autocare.AppRes\"\x00\x12\x43\n\x0fUpdateInference\x12\x16.autocare.InferenceReq\x1a\x16.autocare.InferenceRes\"\x00\x12\x34\n\nInstallApp\x12\x10.autocare.AppReq\x1a\x10.autocare.AppRes\"\x00(\x01\x12\x34\n\x0cUninstallApp\x12\x10.autocare.AppReq\x1a\x10.autocare.AppRes\"\x00\x12\x33\n\nGetAppList\x12\x10.autocare.AppReq\x1a\x11.autocare.AppList\"\x00\x12\x45\n\x10GetInferenceList\x12\x16.autocare.InferenceReq\x1a\x17.autocare.InferenceList\"\x00\x12\x46\n\x12GetInferenceStatus\x12\x16.autocare.InferenceReq\x1a\x16.autocare.InferenceRes\"\x00\x12G\n\x15GetInferenceStatusAll\x12\x10.autocare.AppReq\x1a\x1a.autocare.InferenceResList\"\x00\x12G\n\x13RequestPreviewImage\x12\x16.autocare.InferenceReq\x1a\x16.autocare.InferenceRes\"\x00\x12\x39\n\x11LicenseActivation\x12\x10.autocare.LicReq\x1a\x10.autocare.LicRes\"\x00\x12;\n\x13LicenseDeactivation\x12\x10.autocare.LicReq\x1a\x10.autocare.LicRes\"\x00\x12\x37\n\x0fLicenseActivate\x12\x10.autocare.LicReq\x1a\x10.autocare.LicRes\"\x00\x12\x42\n\x0eStartStreaming\x12\x16.autocare.StreamingReq\x1a\x16.autocare.StreamingRes\"\x00\x12\x41\n\rStopStreaming\x12\x16.autocare.StreamingReq\x1a\x16.autocare.StreamingRes\"\x00\x12(\n\x05GetDx\x12\x0f.autocare.Empty\x1a\x0c.autocare.Dx\"\x00\x42\x0f\xaa\x02\x0cGrpcDxClientb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002\014GrpcDxClient'
  _globals['_EMPTY']._serialized_start=22
  _globals['_EMPTY']._serialized_end=29
  _globals['_PIPELINEREQ']._serialized_start=31
  _globals['_PIPELINEREQ']._serialized_end=60
  _globals['_PIPELINERES']._serialized_start=62
  _globals['_PIPELINERES']._serialized_end=91
  _globals['_INFERENCEREQ']._serialized_start=94
  _globals['_INFERENCEREQ']._serialized_end=284
  _globals['_INFERENCERES']._serialized_start=287
  _globals['_INFERENCERES']._serialized_end=534
  _globals['_APPREQ']._serialized_start=536
  _globals['_APPREQ']._serialized_end=606
  _globals['_APPRES']._serialized_start=608
  _globals['_APPRES']._serialized_end=632
  _globals['_APPLIST']._serialized_start=634
  _globals['_APPLIST']._serialized_end=671
  _globals['_INFERENCELIST']._serialized_start=673
  _globals['_INFERENCELIST']._serialized_end=731
  _globals['_INFERENCERESLIST']._serialized_start=733
  _globals['_INFERENCERESLIST']._serialized_end=794
  _globals['_LICREQ']._serialized_start=796
  _globals['_LICREQ']._serialized_end=868
  _globals['_LICRES']._serialized_start=870
  _globals['_LICRES']._serialized_end=932
  _globals['_STREAMINGREQ']._serialized_start=934
  _globals['_STREAMINGREQ']._serialized_end=1014
  _globals['_STREAMINGRES']._serialized_start=1016
  _globals['_STREAMINGRES']._serialized_end=1142
  _globals['_MODEL']._serialized_start=1145
  _globals['_MODEL']._serialized_end=1357
  _globals['_EVENT']._serialized_start=1360
  _globals['_EVENT']._serialized_end=1517
  _globals['_EVENT_TARGET']._serialized_start=1454
  _globals['_EVENT_TARGET']._serialized_end=1517
  _globals['_PIPELINE']._serialized_start=1519
  _globals['_PIPELINE']._serialized_end=1612
  _globals['_APP']._serialized_start=1615
  _globals['_APP']._serialized_end=1974
  _globals['_DX']._serialized_start=1977
  _globals['_DX']._serialized_end=2154
  _globals['_DETECTOR']._serialized_start=2157
  _globals['_DETECTOR']._serialized_end=3235
# @@protoc_insertion_point(module_scope)



import sys
import os
import typer
from typing_extensions import Annotated
from pathlib import Path
from grpc import RpcError
import asyncio
from functools import wraps
from time import time
from shutil import copyfile

from inference.inference import InferenceDX
from inference.file_writer import FileWriter
from inference.nats_listener import NatsListener

from PyQt5.QtWidgets import QApplication

from player.player import VideoPlayer, text_input_player

app = typer.Typer()

def coro(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        return asyncio.run(f(*args, **kwargs))
    
    return wrapper

@app.command('inference')
@coro
async def cli_inference(file_path: Annotated[Path, typer.Option('-F', '--file', help='Input file path.')],
                  app_path: Annotated[Path, typer.Option('-A', '--app', help='Input app path.')],
                  output_path: Annotated[Path, typer.Option('-O', '--output', help='Store inference result.')]):
    """
    Inference video, save log.
    """
    # 파일 확인
    if not file_path.exists():
        print(f'{file_path} is not exist.')
        raise typer.Exit(-1)
    
    if not app_path.exists():
        print(f'{app_path} is not exist.')
        raise typer.Exit(-1)
    
    if output_path.exists():
        print(f'{output_path} is already exist.')
        raise typer.Exit(-1)

    print(f'input file: {file_path}, app: {app_path}, output file: {output_path}')

    # Inference 시작
    dx_inf = InferenceDX()
    timestamp = int(time())
    
    try:
        dx_inf.ping()
    except RpcError as e:
        print('Start Dx first.')
        raise typer.Exit(-1)
    
    # app id 추출
    app_name = app_path.name
    
    if len(app_name) < 36:
        print('App file must start with uuid.')
        raise typer.Exit(-1)
    
    app_id = app_name[:36]
    
    # install app
    try:
        dx_inf.install_app(app_id, app_path)
    except RpcError as e:
        # 실제로 문제가 있는지, 아니면 app이 이미 인스톨 되어있어 에러가 가는지
        # 확인이 어려워 일단 에러 메시지만 출력하고 넘어감
        print(e)
    
    # video copy
    dx_file_path = Path('/opt/autocare/dx/video').joinpath(file_path.name)
    dx_file_volume_path = Path('/opt/autocare/dx/volume/video').joinpath(file_path.name)
    print(dx_file_path)

    copyfile(file_path, dx_file_path)

    # log writer and nats listener
    log_writer = FileWriter(output_path, file_path, timestamp)
    nats_listener = NatsListener(log_writer)
    await nats_listener.connect()

    write_task = asyncio.create_task(nats_listener.record())
    await asyncio.sleep(0)

    uri = 'file://' + str(dx_file_volume_path)
    print(f'uri: {uri}')
    timestamp = timestamp * 10**6
    print(f'timestamp: {timestamp}')

    # start inference
    try:
        await dx_inf.start_inference(app_id, 'profile', uri, timestamp)
    except RpcError as e:
        print(e)
        dx_inf.clean_up_inference()
        raise typer.Exit(-1)
    except Exception as e:
        print(e)
        dx_inf.clean_up_inference()
        raise typer.Exit(-1)
    
    log_writer.close()
    await asyncio.sleep(0)
    await write_task

    print('Done.')
    

@app.command('profile')
def cli_profile(input_path: Annotated[Path, typer.Option('-F', '--file', help='Input file path.')]):
    """
    Play video with inference log info.
    """
    # 파일 확인
    if not input_path.exists():
        print(f'{input_path} is not exist.')
        raise typer.Exit(-1)
    
    print(f'input file: {input_path}')

    app = QApplication(sys.argv)

    player = text_input_player(input_path)
    
    player.show()
    sys.exit(app.exec_())


@app.callback()
def check_privileged():
    if not os.geteuid() == 0:
        print('Run with root privileged.')
        raise typer.Exit(-1)


if __name__ == '__main__':
    app()



from pathlib import Path

class FileWriter:
    def __init__(self, file: Path, video_path: Path, start_timestamp: int) -> None:
        self.f = open(file, 'w')
        self.f.write(str(video_path) + '\n')
        self.f.write(str(start_timestamp) + '\n')
    
    def write(self, text: str, end: str = '\n') -> None:
        self.f.write(text + end)
    
    def close(self) -> None:
        if not self.f.closed:
            self.f.close()
    
    def is_closed(self) -> bool:
        return self.f.closed

    def __del__(self):
        self.f.close()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import proto_py.dx_pb2 as dx__pb2


class DetectorStub(object):
    """DX API Serivce

    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AddInference = channel.unary_unary(
                '/autocare.Detector/AddInference',
                request_serializer=dx__pb2.InferenceReq.SerializeToString,
                response_deserializer=dx__pb2.InferenceRes.FromString,
                )
        self.RemoveInference = channel.unary_unary(
                '/autocare.Detector/RemoveInference',
                request_serializer=dx__pb2.InferenceReq.SerializeToString,
                response_deserializer=dx__pb2.InferenceRes.FromString,
                )
        self.RemoveInferenceAll = channel.unary_unary(
                '/autocare.Detector/RemoveInferenceAll',
                request_serializer=dx__pb2.AppReq.SerializeToString,
                response_deserializer=dx__pb2.AppRes.FromString,
                )
        self.UpdateInference = channel.unary_unary(
                '/autocare.Detector/UpdateInference',
                request_serializer=dx__pb2.InferenceReq.SerializeToString,
                response_deserializer=dx__pb2.InferenceRes.FromString,
                )
        self.InstallApp = channel.stream_unary(
                '/autocare.Detector/InstallApp',
                request_serializer=dx__pb2.AppReq.SerializeToString,
                response_deserializer=dx__pb2.AppRes.FromString,
                )
        self.UninstallApp = channel.unary_unary(
                '/autocare.Detector/UninstallApp',
                request_serializer=dx__pb2.AppReq.SerializeToString,
                response_deserializer=dx__pb2.AppRes.FromString,
                )
        self.GetAppList = channel.unary_unary(
                '/autocare.Detector/GetAppList',
                request_serializer=dx__pb2.AppReq.SerializeToString,
                response_deserializer=dx__pb2.AppList.FromString,
                )
        self.GetInferenceList = channel.unary_unary(
                '/autocare.Detector/GetInferenceList',
                request_serializer=dx__pb2.InferenceReq.SerializeToString,
                response_deserializer=dx__pb2.InferenceList.FromString,
                )
        self.GetInferenceStatus = channel.unary_unary(
                '/autocare.Detector/GetInferenceStatus',
                request_serializer=dx__pb2.InferenceReq.SerializeToString,
                response_deserializer=dx__pb2.InferenceRes.FromString,
                )
        self.GetInferenceStatusAll = channel.unary_unary(
                '/autocare.Detector/GetInferenceStatusAll',
                request_serializer=dx__pb2.AppReq.SerializeToString,
                response_deserializer=dx__pb2.InferenceResList.FromString,
                )
        self.RequestPreviewImage = channel.unary_unary(
                '/autocare.Detector/RequestPreviewImage',
                request_serializer=dx__pb2.InferenceReq.SerializeToString,
                response_deserializer=dx__pb2.InferenceRes.FromString,
                )
        self.LicenseActivation = channel.unary_unary(
                '/autocare.Detector/LicenseActivation',
                request_serializer=dx__pb2.LicReq.SerializeToString,
                response_deserializer=dx__pb2.LicRes.FromString,
                )
        self.LicenseDeactivation = channel.unary_unary(
                '/autocare.Detector/LicenseDeactivation',
                request_serializer=dx__pb2.LicReq.SerializeToString,
                response_deserializer=dx__pb2.LicRes.FromString,
                )
        self.LicenseActivate = channel.unary_unary(
                '/autocare.Detector/LicenseActivate',
                request_serializer=dx__pb2.LicReq.SerializeToString,
                response_deserializer=dx__pb2.LicRes.FromString,
                )
        self.StartStreaming = channel.unary_unary(
                '/autocare.Detector/StartStreaming',
                request_serializer=dx__pb2.StreamingReq.SerializeToString,
                response_deserializer=dx__pb2.StreamingRes.FromString,
                )
        self.StopStreaming = channel.unary_unary(
                '/autocare.Detector/StopStreaming',
                request_serializer=dx__pb2.StreamingReq.SerializeToString,
                response_deserializer=dx__pb2.StreamingRes.FromString,
                )
        self.GetDx = channel.unary_unary(
                '/autocare.Detector/GetDx',
                request_serializer=dx__pb2.Empty.SerializeToString,
                response_deserializer=dx__pb2.Dx.FromString,
                )


class DetectorServicer(object):
    """DX API Serivce

    """

    def AddInference(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveInference(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveInferenceAll(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateInference(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InstallApp(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UninstallApp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAppList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInferenceList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInferenceStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInferenceStatusAll(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RequestPreviewImage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LicenseActivation(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LicenseDeactivation(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LicenseActivate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartStreaming(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopStreaming(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDx(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DetectorServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'AddInference': grpc.unary_unary_rpc_method_handler(
                    servicer.AddInference,
                    request_deserializer=dx__pb2.InferenceReq.FromString,
                    response_serializer=dx__pb2.InferenceRes.SerializeToString,
            ),
            'RemoveInference': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveInference,
                    request_deserializer=dx__pb2.InferenceReq.FromString,
                    response_serializer=dx__pb2.InferenceRes.SerializeToString,
            ),
            'RemoveInferenceAll': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveInferenceAll,
                    request_deserializer=dx__pb2.AppReq.FromString,
                    response_serializer=dx__pb2.AppRes.SerializeToString,
            ),
            'UpdateInference': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateInference,
                    request_deserializer=dx__pb2.InferenceReq.FromString,
                    response_serializer=dx__pb2.InferenceRes.SerializeToString,
            ),
            'InstallApp': grpc.stream_unary_rpc_method_handler(
                    servicer.InstallApp,
                    request_deserializer=dx__pb2.AppReq.FromString,
                    response_serializer=dx__pb2.AppRes.SerializeToString,
            ),
            'UninstallApp': grpc.unary_unary_rpc_method_handler(
                    servicer.UninstallApp,
                    request_deserializer=dx__pb2.AppReq.FromString,
                    response_serializer=dx__pb2.AppRes.SerializeToString,
            ),
            'GetAppList': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAppList,
                    request_deserializer=dx__pb2.AppReq.FromString,
                    response_serializer=dx__pb2.AppList.SerializeToString,
            ),
            'GetInferenceList': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInferenceList,
                    request_deserializer=dx__pb2.InferenceReq.FromString,
                    response_serializer=dx__pb2.InferenceList.SerializeToString,
            ),
            'GetInferenceStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInferenceStatus,
                    request_deserializer=dx__pb2.InferenceReq.FromString,
                    response_serializer=dx__pb2.InferenceRes.SerializeToString,
            ),
            'GetInferenceStatusAll': grpc.unary_unary_rpc_method_handler(
                    servicer.GetInferenceStatusAll,
                    request_deserializer=dx__pb2.AppReq.FromString,
                    response_serializer=dx__pb2.InferenceResList.SerializeToString,
            ),
            'RequestPreviewImage': grpc.unary_unary_rpc_method_handler(
                    servicer.RequestPreviewImage,
                    request_deserializer=dx__pb2.InferenceReq.FromString,
                    response_serializer=dx__pb2.InferenceRes.SerializeToString,
            ),
            'LicenseActivation': grpc.unary_unary_rpc_method_handler(
                    servicer.LicenseActivation,
                    request_deserializer=dx__pb2.LicReq.FromString,
                    response_serializer=dx__pb2.LicRes.SerializeToString,
            ),
            'LicenseDeactivation': grpc.unary_unary_rpc_method_handler(
                    servicer.LicenseDeactivation,
                    request_deserializer=dx__pb2.LicReq.FromString,
                    response_serializer=dx__pb2.LicRes.SerializeToString,
            ),
            'LicenseActivate': grpc.unary_unary_rpc_method_handler(
                    servicer.LicenseActivate,
                    request_deserializer=dx__pb2.LicReq.FromString,
                    response_serializer=dx__pb2.LicRes.SerializeToString,
            ),
            'StartStreaming': grpc.unary_unary_rpc_method_handler(
                    servicer.StartStreaming,
                    request_deserializer=dx__pb2.StreamingReq.FromString,
                    response_serializer=dx__pb2.StreamingRes.SerializeToString,
            ),
            'StopStreaming': grpc.unary_unary_rpc_method_handler(
                    servicer.StopStreaming,
                    request_deserializer=dx__pb2.StreamingReq.FromString,
                    response_serializer=dx__pb2.StreamingRes.SerializeToString,
            ),
            'GetDx': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDx,
                    request_deserializer=dx__pb2.Empty.FromString,
                    response_serializer=dx__pb2.Dx.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'autocare.Detector', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Detector(object):
    """DX API Serivce

    """

    @staticmethod
    def AddInference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/AddInference',
            dx__pb2.InferenceReq.SerializeToString,
            dx__pb2.InferenceRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveInference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/RemoveInference',
            dx__pb2.InferenceReq.SerializeToString,
            dx__pb2.InferenceRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveInferenceAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/RemoveInferenceAll',
            dx__pb2.AppReq.SerializeToString,
            dx__pb2.AppRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateInference(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/UpdateInference',
            dx__pb2.InferenceReq.SerializeToString,
            dx__pb2.InferenceRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def InstallApp(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/autocare.Detector/InstallApp',
            dx__pb2.AppReq.SerializeToString,
            dx__pb2.AppRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UninstallApp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/UninstallApp',
            dx__pb2.AppReq.SerializeToString,
            dx__pb2.AppRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAppList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/GetAppList',
            dx__pb2.AppReq.SerializeToString,
            dx__pb2.AppList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetInferenceList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/GetInferenceList',
            dx__pb2.InferenceReq.SerializeToString,
            dx__pb2.InferenceList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetInferenceStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/GetInferenceStatus',
            dx__pb2.InferenceReq.SerializeToString,
            dx__pb2.InferenceRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetInferenceStatusAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/GetInferenceStatusAll',
            dx__pb2.AppReq.SerializeToString,
            dx__pb2.InferenceResList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RequestPreviewImage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/RequestPreviewImage',
            dx__pb2.InferenceReq.SerializeToString,
            dx__pb2.InferenceRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LicenseActivation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/LicenseActivation',
            dx__pb2.LicReq.SerializeToString,
            dx__pb2.LicRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LicenseDeactivation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/LicenseDeactivation',
            dx__pb2.LicReq.SerializeToString,
            dx__pb2.LicRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LicenseActivate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/LicenseActivate',
            dx__pb2.LicReq.SerializeToString,
            dx__pb2.LicRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartStreaming(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/StartStreaming',
            dx__pb2.StreamingReq.SerializeToString,
            dx__pb2.StreamingRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopStreaming(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/StopStreaming',
            dx__pb2.StreamingReq.SerializeToString,
            dx__pb2.StreamingRes.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDx(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/autocare.Detector/GetDx',
            dx__pb2.Empty.SerializeToString,
            dx__pb2.Dx.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
